Exception: NoMultisamplesInDirectoryException

Inherits:
Exception
  • Object
show all
Defined in:
lib/DirectoryProcessor.rb

Overview

Exception to be raised in the case that no Multisample directories Have been found by DirectoryProcessor.

Instance Method Summary collapse

Instance Method Details

#messageObject



239
240
241
# File 'lib/DirectoryProcessor.rb', line 239

def message
	"No multisample directories were found in the given directories."
end